@@ -1,8 +1,6 @@
 """Library to (partially) validate a Concrete Communication
 
 Current validation checks:
-  - do all sentenceSegmentation.sectionId's match the uuid of the
-    enclosing section?
   - for each constituent parse, do any of the constituent ID's for
     that parse repeat?
   - is each dependency and constituent parse a fully connected graph?
@@ -50,31 +48,16 @@ def validate_communication(comm):
 
     valid &= validate_thrift_object_required_fields_recursively(comm)
 
-    if comm.sectionSegmentations:
-        logging.debug(ilm(1, "Communication '%s' has %d sectionSegmentations" %
-                          (comm.id, len(comm.sectionSegmentations))))
-        for sectionSegmentation in comm.sectionSegmentations:
-            logging.debug(ilm(2, "sectionSegmentation '%s' has %d sections" %
-                              (sectionSegmentation.uuid, len(sectionSegmentation.sectionList))))
-            for section in sectionSegmentation.sectionList:
-                if section.sentenceSegmentation:
-                    valid &= validate_token_offsets_for_section(section)
-                    logging.debug(ilm(3, "section '%s' has %d sentenceSegmentations" %
-                                      (section.uuid, len(section.sentenceSegmentation))))
-                    for sentenceSegmentation in section.sentenceSegmentation:
-                        logging.debug(ilm(4, "sentenceSegmentation '%s' has %d sentences" %
-                                          (sentenceSegmentation.uuid, len(sentenceSegmentation.sentenceList))))
-                        if sentenceSegmentation.sectionId != section.uuid:
-                            valid = False
-                            logging.error(ilm(5, "sentenceSegmentation.sectionId '%s' does not match section.uuid '%s'" %
-                                              (sentenceSegmentation.sectionId, sentenceSegmentation.uuid)))
-                        for sentence in sentenceSegmentation.sentenceList:
-                            logging.debug(ilm(5, "sentence '%s' has %d tokenizations" %
-                                              (sentence.uuid, len(sentence.tokenizationList))))
-                            valid &= validate_token_offsets_for_sentence(sentence)
-                            for tokenization in sentence.tokenizationList:
-                                valid &= validate_constituency_parse(comm, tokenization)
-                                valid &= validate_dependency_parses(tokenization)
+    if comm.sectionList:
+        for section in comm.sectionList:
+            valid &= validate_token_offsets_for_section(section)
+            if section.sentenceList:
+                logging.debug(ilm(4, "section '%s' has %d sentences" %
+                                  (section.uuid, len(section.sentenceList))))
+                for sentence in section.sentenceList:
+                    valid &= validate_token_offsets_for_sentence(sentence)
+                    valid &= validate_constituency_parses(comm, sentence.tokenization)
+                    valid &= validate_dependency_parses(sentence.tokenization)
 
     valid &= validate_entity_mention_ids(comm)
     valid &= validate_entity_mention_tokenization_ids(comm)
@@ -99,8 +82,8 @@ def get_entity_uuidString_set(comm):
       set of strings: uuidStrings for all Entities in the Communication
     """
     entity_uuidString_set = set()
-    if comm.entitySets:
-        for entitySet in comm.entitySets:
+    if comm.entitySetList:
+        for entitySet in comm.entitySetList:
             if entitySet.entityList:
                 for entity in entitySet.entityList:
                     entity_uuidString_set.add(entity.uuid.uuidString)
@@ -116,14 +99,33 @@ def get_entity_mention_uuidString_set(comm):
       set of strings: uuidStrings for all EntityMentions in the Communication
     """
     entity_mention_uuidString_set = set()
-    if comm.entityMentionSets:
-        for entityMentionSet in comm.entityMentionSets:
-            if entityMentionSet.mentionSet:
-                for entityMention in entityMentionSet.mentionSet:
+    if comm.entityMentionSetList:
+        for entityMentionSet in comm.entityMentionSetList:
+            if entityMentionSet.mentionList:
+                for entityMention in entityMentionSet.mentionList:
                     entity_mention_uuidString_set.add(entityMention.uuid.uuidString)
     return entity_mention_uuidString_set
 
 
+def get_sentence_for_tokenization_uuidString_dict(comm):
+    """
+    Args:
+      comm (concrete.structure.ttypes.Communication)
+
+    Returns:
+      dictionary mapping of Tokenization uuidStrings to Sentences
+    """
+    if not hasattr(comm, 'sentence_for_tokenization_uuidString_dict'):
+        comm.sentence_for_tokenization_uuidString_dict = {}
+        if comm.sectionList:
+            for section in comm.sectionList:
+                if section.sentenceList:
+                    for sentence in section.sentenceList:
+                        if sentence.tokenization:
+                            comm.sentence_for_tokenization_uuidString_dict[sentence.tokenization.uuid.uuidString] = sentence
+    return comm.sentence_for_tokenization_uuidString_dict
+
+
 def get_situation_uuidString_set(comm):
     """
     Args:
@@ -133,8 +135,8 @@ def get_situation_uuidString_set(comm):
       set of strings: uuidStrings for all Situations in the Communication
     """
     situation_uuidString_set = set()
-    if comm.situationSets:
-        for situationSet in comm.situationSets:
+    if comm.situationSetList:
+        for situationSet in comm.situationSetList:
             if situationSet.situationList:
                 for situation in situationSet.situationList:
                     situation_uuidString_set.add(situation.uuid.uuidString)
@@ -150,8 +152,8 @@ def get_situation_mention_uuidString_set(comm):
       set of strings: uuidStrings for all SituationMentions in the Communication
     """
     situation_mention_uuidString_set = set()
-    if comm.situationMentionSets:
-        for situationMentionSet in comm.situationMentionSets:
+    if comm.situationMentionSetList:
+        for situationMentionSet in comm.situationMentionSetList:
             if situationMentionSet.mentionList:
                 for situationMention in situationMentionSet.mentionList:
                     situation_mention_uuidString_set.add(situationMention.uuid.uuidString)
@@ -168,14 +170,12 @@ def get_tokenization_uuidString_dict(comm):
     """
     if not hasattr(comm, '_tokenization_uuidString_dict'):
         comm._tokenization_uuidString_dict = {}
-        if comm.sectionSegmentations:
-            for sectionSegmentation in comm.sectionSegmentations:
-                for section in sectionSegmentation.sectionList:
-                    if section.sentenceSegmentation:
-                        for sentenceSegmentation in section.sentenceSegmentation:
-                            for sentence in sentenceSegmentation.sentenceList:
-                                for tokenization in sentence.tokenizationList:
-                                    comm._tokenization_uuidString_dict[tokenization.uuid.uuidString] = tokenization
+        if comm.sectionList:
+            for section in comm.sectionList:
+                if section.sentenceList:
+                    for sentence in section.sentenceList:
+                        if sentence.tokenization:
+                            comm._tokenization_uuidString_dict[sentence.tokenization.uuid.uuidString] = sentence.tokenization
     return comm._tokenization_uuidString_dict
 
 
@@ -188,14 +188,12 @@ def get_tokenization_uuidString_set(comm):
       set of strings: uuidStrings for all Tokenizations in the Communication
     """
     tokenization_uuidString_set = set()
-    if comm.sectionSegmentations:
-        for sectionSegmentation in comm.sectionSegmentations:
-            for section in sectionSegmentation.sectionList:
-                if section.sentenceSegmentation:
-                    for sentenceSegmentation in section.sentenceSegmentation:
-                        for sentence in sentenceSegmentation.sentenceList:
-                            for tokenization in sentence.tokenizationList:
-                                tokenization_uuidString_set.add(tokenization.uuid.uuidString)
+    if comm.sectionList:
+        for section in comm.sectionList:
+            if section.sentenceList:
+                for sentence in section.sentenceList:
+                    if sentence.tokenization:
+                        tokenization_uuidString_set.add(sentence.tokenization.uuid.uuidString)
     return tokenization_uuidString_set
 
 
@@ -216,7 +214,7 @@ def ilm(indent_level, log_message):
     return "  " * indent_level + log_message
 
 
-def validate_constituency_parse(comm, tokenization):
+def validate_constituency_parses(comm, tokenization):
     """
     Args:
       tokenization (concrete.structure.ttypes.Tokenization)
@@ -226,55 +224,56 @@ def validate_constituency_parse(comm, tokenization):
     """
     valid = True
 
-    if tokenization.parse:
-        total_constituents = len(tokenization.parse.constituentList)
-        logging.debug(ilm(6, "tokenization '%s' has %d constituents" % (tokenization.uuid, total_constituents)))
+    if tokenization.parseList:
+        for parse in tokenization.parseList:
+            total_constituents = len(parse.constituentList)
+            logging.debug(ilm(6, "tokenization '%s' has %d constituents" % (tokenization.uuid, total_constituents)))
 
-        total_uuid_mismatches = 0
-        constituent_id_set = set()
-        constituent_parse_tree = nx.DiGraph()
+            total_uuid_mismatches = 0
+            constituent_id_set = set()
+            constituent_parse_tree = nx.DiGraph()
 
-        for constituent in tokenization.parse.constituentList:
-            # Add nodes to parse tree
-            constituent_parse_tree.add_node(constituent.id)
+            for constituent in parse.constituentList:
+                # Add nodes to parse tree
+                constituent_parse_tree.add_node(constituent.id)
 
-            if constituent.id not in constituent_id_set:
-                constituent_id_set.add(constituent.id)
-            else:
+                if constituent.id not in constituent_id_set:
+                    constituent_id_set.add(constituent.id)
+                else:
+                    valid = False
+                    logging.error(ilm(7, "constituent ID %d has already been used in this sentence's tokenization" % constituent.id))
+
+                # Per the Concrete 'structure.thrift' file, tokenSequence may not be defined:
+                #   "Typically, this field will only be defined for leaf constituents (i.e., constituents with no children)."
+                if constituent.tokenSequence and constituent.tokenSequence.tokenizationId != tokenization.uuid:
+                    total_uuid_mismatches += 1
+
+                if constituent.tokenSequence:
+                    valid &= validate_token_ref_sequence(comm, constituent.tokenSequence)
+
+            if total_uuid_mismatches > 0:
                 valid = False
-                logging.error(ilm(7, "constituent ID %d has already been used in this sentence's tokenization" % constituent.id))
+                logging.error(ilm(6, "tokenization '%s' has UUID mismatch for %d/%d constituents" %
+                                  (tokenization.uuid, total_uuid_mismatches, total_constituents)))
 
-            # Per the Concrete 'structure.thrift' file, tokenSequence may not be defined:
-            #   "Typically, this field will only be defined for leaf constituents (i.e., constituents with no children)."
-            if constituent.tokenSequence and constituent.tokenSequence.tokenizationId != tokenization.uuid:
-                total_uuid_mismatches += 1
+            # Add edges to constituent parse tree
+            for constituent in parse.constituentList:
+                if constituent.childList:
+                    for child_id in constituent.childList:
+                        constituent_parse_tree.add_edge(constituent.id, child_id)
 
-            if constituent.tokenSequence:
-                valid &= validate_token_ref_sequence(comm, constituent.tokenSequence)
+            # Check if constituent parse "tree" is a fully connected graph
+            undirected_graph = constituent_parse_tree.to_undirected()
+            if not nx.is_connected(undirected_graph):
+                valid = False
+                logging.error(ilm(6, "The constituent parse \"tree\" is not a fully connected graph - the graph has %d components" %
+                    len(nx.connected_components(undirected_graph))))
 
-        if total_uuid_mismatches > 0:
-            valid = False
-            logging.error(ilm(6, "tokenization '%s' has UUID mismatch for %d/%d constituents" %
-                              (tokenization.uuid, total_uuid_mismatches, total_constituents)))
-
-        # Add edges to constituent parse tree
-        for constituent in tokenization.parse.constituentList:
-            if constituent.childList:
-                for child_id in constituent.childList:
-                    constituent_parse_tree.add_edge(constituent.id, child_id)
-
-        # Check if constituent parse "tree" is a fully connected graph
-        undirected_graph = constituent_parse_tree.to_undirected()
-        if not nx.is_connected(undirected_graph):
-            valid = False
-            logging.error(ilm(6, "The constituent parse \"tree\" is not a fully connected graph - the graph has %d components" %
-                len(nx.connected_components(undirected_graph))))
-
-        # Check if constituent parse "tree" is actually a tree
-        if nx.number_of_nodes(constituent_parse_tree) != nx.number_of_edges(constituent_parse_tree) + 1:
-            valid = False
-            logging.error(ilm(6, "The constituent parse \"tree\" is not a tree.  |V| != |E|+1  (|V|=%d, |E|=%d)" %
-                (nx.number_of_nodes(constituent_parse_tree), nx.number_of_edges(constituent_parse_tree))))
+            # Check if constituent parse "tree" is actually a tree
+            if nx.number_of_nodes(constituent_parse_tree) != nx.number_of_edges(constituent_parse_tree) + 1:
+                valid = False
+                logging.error(ilm(6, "The constituent parse \"tree\" is not a tree.  |V| != |E|+1  (|V|=%d, |E|=%d)" %
+                    (nx.number_of_nodes(constituent_parse_tree), nx.number_of_edges(constituent_parse_tree))))
 
     return valid
 
@@ -324,8 +323,8 @@ def validate_entity_mention_ids(comm):
     valid = True
     entity_mention_uuidString_set = get_entity_mention_uuidString_set(comm)
 
-    if comm.entitySets:
-        for entitySet in comm.entitySets:
+    if comm.entitySetList:
+        for entitySet in comm.entitySetList:
             if entitySet.entityList:
                 for entity in entitySet.entityList:
                     for entityMentionId in entity.mentionIdList:
@@ -339,10 +338,10 @@ def validate_entity_mention_tokenization_ids(comm):
     valid = True
     tokenization_uuidString_set = get_tokenization_uuidString_set(comm)
 
-    if comm.entityMentionSets:
-        for entityMentionSet in comm.entityMentionSets:
-            if entityMentionSet.mentionSet:
-                for entityMention in entityMentionSet.mentionSet:
+    if comm.entityMentionSetList:
+        for entityMentionSet in comm.entityMentionSetList:
+            if entityMentionSet.mentionList:
+                for entityMention in entityMentionSet.mentionList:
                     if entityMention.tokens.tokenizationId.uuidString not in tokenization_uuidString_set:
                         valid = False
                         logging.error(ilm(2, "Mention '%s' has an invalid tokenizationId (%s)" % (entityMention.uuid, entityMention.tokens.tokenizationId)))
@@ -351,10 +350,10 @@ def validate_entity_mention_tokenization_ids(comm):
 
 def validate_entity_mention_token_ref_sequences(comm):
     valid = True
-    if comm.entityMentionSets:
-        for entityMentionSet in comm.entityMentionSets:
-            if entityMentionSet.mentionSet:
-                for entityMention in entityMentionSet.mentionSet:
+    if comm.entityMentionSetList:
+        for entityMentionSet in comm.entityMentionSetList:
+            if entityMentionSet.mentionList:
+                for entityMention in entityMentionSet.mentionList:
                     valid &= validate_token_ref_sequence(comm, entityMention.tokens)
     return valid
 
@@ -364,8 +363,8 @@ def validate_situation_mentions(comm):
     entity_mention_uuidString_set = get_entity_mention_uuidString_set(comm)
     situation_mention_uuidString_set = get_situation_mention_uuidString_set(comm)
 
-    if comm.situationMentionSets:
-        for situationMentionSet in comm.situationMentionSets:
+    if comm.situationMentionSetList:
+        for situationMentionSet in comm.situationMentionSetList:
             if situationMentionSet.mentionList:
                 for situationMention in situationMentionSet.mentionList:
                     if situationMention.tokens:
@@ -391,8 +390,8 @@ def validate_situations(comm):
     situation_mention_uuidString_set = get_situation_mention_uuidString_set(comm)
     situation_uuidString_set = get_situation_uuidString_set(comm)
 
-    if comm.situationSets:
-        for situationSet in comm.situationSets:
+    if comm.situationSetList:
+        for situationSet in comm.situationSetList:
             if situationSet.situationList:
                 for situation in situationSet.situationList:
                     if situation.argumentList:
@@ -413,8 +412,9 @@ def validate_situations(comm):
                                 valid = False
                                 logging.error(ilm(2, "Justification for Situation '%s' has an invalid [situation] mentionId (%s). Tool='%s'" %
                                                   (situation.uuid, justification.mentionId, situationSet.metadata.tool)))
-                            if justification.tokens:
-                                valid &= validate_token_ref_sequence(comm, justification.tokens)
+                            if justification.tokenRefSeqList:
+                                for tokenRefSeq in tokenRefSeqList:
+                                    valid &= validate_token_ref_sequence(comm, tokenRefSeq)
                     if situation.mentionIdList:
                         for mentionId in situation.mentionIdList:
                             if mentionId.uuidString not in situation_mention_uuidString_set:
@@ -439,22 +439,21 @@ def validate_token_offsets_for_section(section):
         logging.error(ilm(2, "Section '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
                           (section.uuid, section.textSpan.start, section.textSpan.ending)))
 
-    if section.sentenceSegmentation:
-        for sentenceSegmentation in section.sentenceSegmentation:
-            for sentence in sentenceSegmentation.sentenceList:
-                if sentence.textSpan == None:
-                    continue
-                if sentence.textSpan.start > sentence.textSpan.ending:
-                    valid = False
-                    logging.error(ilm(2, "Sentence '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
-                                      (sentence.uuid, sentence.textSpan.start, sentence.textSpan.ending)))
-                elif (sentence.textSpan.start < section.textSpan.start) or \
-                     (sentence.textSpan.start > section.textSpan.ending) or \
-                     (sentence.textSpan.ending < section.textSpan.start) or \
-                     (sentence.textSpan.ending > section.textSpan.ending):
-                    valid = False
-                    logging.error(ilm(2, "Sentence '%s' in Section '%s' has a TextSpan [%d, %d] that does not fit within the Section TextSpan [%d, %d]" %
-                                      (sentence.uuid, section.uuid, sentence.textSpan.start, sentence.textSpan.ending, section.textSpan.start, section.textSpan.ending)))
+    if section.sentenceList:
+        for sentence in section.sentenceList:
+            if sentence.textSpan == None:
+                continue
+            if sentence.textSpan.start > sentence.textSpan.ending:
+                valid = False
+                logging.error(ilm(2, "Sentence '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
+                                  (sentence.uuid, sentence.textSpan.start, sentence.textSpan.ending)))
+            elif (sentence.textSpan.start < section.textSpan.start) or \
+                 (sentence.textSpan.start > section.textSpan.ending) or \
+                 (sentence.textSpan.ending < section.textSpan.start) or \
+                 (sentence.textSpan.ending > section.textSpan.ending):
+                valid = False
+                logging.error(ilm(2, "Sentence '%s' in Section '%s' has a TextSpan [%d, %d] that does not fit within the Section TextSpan [%d, %d]" %
+                                  (sentence.uuid, section.uuid, sentence.textSpan.start, sentence.textSpan.ending, section.textSpan.start, section.textSpan.ending)))
 
     return valid
 
@@ -473,8 +472,8 @@ def validate_token_offsets_for_sentence(sentence):
         valid = False
         logging.error(ilm(7, "Sentence '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
                           (sentence.uuid, sentence.textSpan.start, sentence.textSpan.ending)))
-    for tokenization in sentence.tokenizationList:
-        for token in tokenization.tokenList.tokens:
+    if sentence.tokenization:
+        for token in sentence.tokenization.tokenList.tokenList:
             if token.textSpan == None:
                 continue
             if token.textSpan.start > token.textSpan.ending:
@@ -496,20 +495,31 @@ def validate_token_ref_sequence(comm, token_ref_sequence):
     valid = True
 
     tokenization_mapping = get_tokenization_uuidString_dict(comm)
+    sentence_for_tokenization_mapping = get_sentence_for_tokenization_uuidString_dict(comm)
 
     if token_ref_sequence.tokenizationId.uuidString not in tokenization_mapping:
         valid = False
-        logging.error(ilm(3, "TokenRefSequence '%s' has an invalid tokenizationId (%s)" %
-                          (token_ref_sequence.uuid.uuidString, token_ref_sequence.tokenizationId.uuidString)))
+        logging.error(ilm(3, "TokenRefSequence has an invalid tokenizationId (%s)" %
+                          token_ref_sequence.tokenizationId.uuidString))
     else:
         tokenization = tokenization_mapping[token_ref_sequence.tokenizationId.uuidString]
         for tokenIndex in token_ref_sequence.tokenIndexList:
             try:
-                tokenization.tokenList.tokens[tokenIndex]
+                tokenization.tokenList.tokenList[tokenIndex]
             except IndexError:
                 valid = False
                 logging.error(ilm(3, "TokenRefSequence '%s' has an invalid tokenIndex (%d)" %
                                   (token_ref_sequence.tokenizationId.uuidString, tokenIndex)))
+    if token_ref_sequence.tokenizationId.uuidString in sentence_for_tokenization_mapping:
+        sentence = sentence_for_tokenization_mapping[token_ref_sequence.tokenizationId.uuidString]
+        if sentence.textSpan and token_ref_sequence.textSpan:
+            if (token_ref_sequence.textSpan.start < sentence.textSpan.start) or \
+               (token_ref_sequence.textSpan.start > sentence.textSpan.ending) or \
+               (token_ref_sequence.textSpan.ending < sentence.textSpan.start) or \
+               (token_ref_sequence.textSpan.ending > sentence.textSpan.ending):
+                valid = False
+                logging.error(ilm(2, "TokenRefSequence has a TextSpan [%d, %d] that does not fit within the Sentence TextSpan [%d, %d]" %
+                                  (token_ref_sequence.textSpan.start, token_ref_sequence.textSpan.ending, sentence.textSpan.start, sentence.textSpan.ending)))
     return valid
 
 
