@@ -3,9 +3,10 @@
 """
 """
 
+import time
 import unittest
 
-from concrete import Communication
+from concrete import AnnotationMetadata, Communication
 from concrete.util import generate_UUID
 from concrete.validate import validate_communication
 
@@ -14,9 +15,10 @@ class TestUUID(unittest.TestCase):
         comm = Communication()
         comm.uuid = generate_UUID()
 
-    def test_minimial_communication_with_uuid(self):
+    def test_minimal_communication_with_uuid(self):
         comm = Communication()
         comm.id = "myID"
+        comm.metadata = AnnotationMetadata(tool="TEST", timestamp=int(time.time()))
         comm.type = "Test Communication"
         comm.uuid = generate_UUID()
         self.assertTrue(validate_communication(comm))
